#include "CppDir.hh"

#ifdef LINUX
#  include <sys/stat.h>
#  include <unistd.h>
extern "C" { int lstat(const char *file_name, struct stat *buf); }
#endif

std::string CppDir::concat_dir( std::string path, std::string name )
{
#ifdef LINUX
  if( name.size() > 0 )
    if( name[0] == '/' )
      name = name.substr( 1 );

  if( path.size() > 0 )
    if( path[path.size()-1] == '/' )
      path = path.substr( 0, path.size() - 1 );

  path += "/" + name;

  std::string::size_type pos = 0;

  while( true )
    {
      pos = path.find( "/./" );
      if( pos == std::string::npos )
	break;

      path = path.substr( 0, pos ) + path.substr( pos + 2 );
    } 

  return path;
#endif
}

CppDir::File::File( struct dirent d, std::string path)
{
  inode_number = d.d_ino;
  name = d.d_name;
  link = false;

  type = get_type( concat_dir( path, name ) );

  valid = true;
}

CppDir::File::Type CppDir::File::get_type( const std::string& name )
{
#ifdef LINUX
  struct stat stat_buf;

  if( lstat( name.c_str(), &stat_buf ) == -1 )
    return UNKNOWN;

  if( S_ISLNK( stat_buf.st_mode ) )
    link = true;

  if( stat( name.c_str(), &stat_buf ) == -1 )
    return UNKNOWN;

  if( S_ISREG( stat_buf.st_mode ) )
    return REGULAR;

  if( S_ISDIR( stat_buf.st_mode ) )
    return DIR;
    
  if( S_ISCHR( stat_buf.st_mode ) )
    return CHAR;

  if( S_ISBLK( stat_buf.st_mode ) )
    return BLOCK;

  if( S_ISFIFO( stat_buf.st_mode ) )
    return FIFO;

/* //shit BSD specific options 
  if( S_ISSOCK( stat_buf.st_mode ) )
    return SOCKET;
*/
  if( link )
    return LINK;

#endif

  return UNKNOWN;
}

CppDir::Directory::Directory( std::string pname )
{
  is_open = false;
//  readed = false;

  valid = open( pname );
}


CppDir::Directory::Directory( File file )
{
  is_open = false;
//  readed = false;

  valid = open( file );
}

bool CppDir::Directory::open( File file )
{
  if( is_open )
    return false;

  if( !file )
    return false;

  if( file.get_type() != File::DIR )
    return false;

  return open( file.get_name() );
}

bool CppDir::Directory::open( std::string fname )
{
  if( is_open )
    return false;

  if( (dir = opendir( fname.c_str() )) == 0 )
    return false;

  name = fname;

  while( true )
    {
      struct dirent* d = readdir( dir );
      if( d == 0 )
	break;
     
      files.push_back( File( *d , fname ));
    }

//  bool readed = true;

  close();

  return true;
}

void CppDir::Directory::close()
{
  if( is_open )
    {
      closedir( dir );
      is_open = false;
    }
}

ostream& CppDir::operator << ( ostream& out , File::Type type )
{
  switch( type )
    {
#ifdef LINUX
    case File::UNKNOWN: out << "UNKNOWN"; break;
    case File::FIFO   : out << "FIFO"   ; break;
    case File::CHAR   : out << "CHAR"   ; break;
    case File::DIR    : out << "DIR"    ; break;
    case File::BLOCK  : out << "BLOCK"  ; break;
    case File::REGULAR: out << "REGULAR"; break;
    case File::LINK   : out << "LINK"   ; break;
//    case File::SOCKET : out << "SOCKET "; break;
//    case File::WHT    : out << "WHT    "; break;
#endif
    }
  return out;
}
