#ifndef CppDir_hh
#define CppDir_hh

#include <string>
#include <list>

#ifdef LINUX
#  include <dirent.h>
#  include <sys/types.h>
#endif

class ostream;

/** This is a simple C++ interface for accessing 
    files and directories. It's a fast hack
    and won't be developed today as an extra package.
*/

namespace CppDir 
{
  class File ///< A simple file
  {
  public:
#ifdef LINUX
    typedef enum Type ///< possible file types
      {
	UNKNOWN  =  0, ///< unknown file type
	FIFO     =  1, ///< fifo
	CHAR     =  2, ///< character device 
	DIR      =  4, ///< directory
	BLOCK    =  6, ///< blockdevice
	REGULAR  =  8, ///< regular file
	LINK     = 10  ///< link
//	SOCKET   = 12  ///< socket
      };
#endif
    
  private:    
    std::string name; ///< name of the file
    Type type;        ///< file type

    long inode_number; ///< inode number
    bool valid;        ///< is a valid file?
    bool link;
    std::string path;  
  public:
#ifdef LINUX
    /** extract file informations from the dirent structure */
    File( struct dirent d, std::string path ); 
#endif

    bool is_valid() const { return valid; }
    bool operator!() const { return !valid; }

    std::string get_name() const { return name; }
    Type        get_type() const { return type; }
    long        get_inode() const { return inode_number; }
    bool        is_link() const { return link; }
    std::string get_path() const { return path; }
  private:
    Type get_type( const std::string& name );
    };
    
  //-------------------

  class Directory ///< class representing a directory
    {
    public:
      typedef std::list<File> file_list; 
      typedef file_list::iterator file_list_it; 

    private:
      std::string name; ///< directory name
      bool valid;       ///< true if the directory is valid
      file_list files;  ///< list of file withen the directory
      bool is_open;     ///< true if the directory is open
//      bool readed;
      
#ifdef LINUX
      DIR* dir;  ///< directory HANDLE
#endif
    public:
      Directory( std::string pname );
      Directory( File file );

      bool is_valid() const { return valid; }
      bool operator!() const { return !valid; }

      /** returns the list of files that the directory contains */
      file_list get_files() { return files; }

    private:
      bool open( File file ); ///< opens the directory
      bool open( std::string fname ); ///< opens the directory
      void close(); ///< closes the directory
    };

  ostream& operator << ( ostream& out , File::Type type );
  std::string concat_dir( std::string path, std::string name );
}

#endif
